
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"

#IFDEF UseDXE
#include "DXE_CLONE.CH"
#ENDIF

#DEFINE CRLF Chr(13)+Chr(10)

FUNCTION IniTreeForm(cIni, bClose, bOk, bCancel)
LOCAL oDlg := IniTree():new()
LOCAL bDef := {|| NIL }
LOCAL oIni := TIniFile():new(cIni)
LOCAL aSections := {}
LOCAL i
LOCAL oRoot
LOCAL oItem

DEFAULT bClose to bDef,;
        bOk to bDef,;
        bCancel to bDef

   oDlg:close             := bClose
   oDlg:pbOk:activate     := bOk
   oDlg:pbCancel:activate := bCancel

   oDlg:title := cIni

   oDlg:tv:alwaysShowSelection := TRUE
   oDlg:tv:hasButtons          := TRUE
   oDlg:tv:hasLines            := TRUE
   oDlg:tv:itemMarked          := {|oItem,Rect,self| oDlg:upDateControls(oItem) }
   oDlg:sleValue:keyboard      := {|nKey| iif(nKey == xbeK_ENTER .or. nKey == xbeK_RETURN, oDlg:iSetItem(), NIL) }
   oDlg:mleSection:lbClick     := {|aPos,uNil,self| oDlg:mleClick(aPos) }
   oDlg:maxbutton              := FALSE

   oDlg:create()
   oDlg:maxSize                := oDlg:currentSize()
   oDlg:minSize                := oDlg:currentSize()

   oDlg:oIni := oIni

   oDlg:oIni:readSections(aSections)
   oRoot := oDlg:tv:rootItem

   oDlg:tv:lockUpdate(.T.)
   for i := 1 to len(aSections)

//*      oItem := XbpTreeViewItem():new()
//*      oItem:caption := aSections[i]
//*      oItem:create()
//*      oItem := oRoot:addItem( oItem )

      oItem := oRoot:addItem( aSections[i] )

      oDlg:getItemsFromSection( aSections[i], oItem )
   next
   oDlg:tv:lockUpdate(.F.)

   oDlg:cArchive := MemoRead(cIni)

RETURN (oDlg)


CLASS IniTree FROM _IniTree
   EXPORTED:
      VAR oIni
      VAR oCMenu
      VAR cArchive

      METHOD init
      METHOD create
      METHOD getItemsFromSection
      METHOD updateControls
      METHOD set
      METHOD iSetItem
      METHOD mleClick
      METHOD nodeDelete
      METHOD undo
ENDCLASS

METHOD IniTree:init( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::_IniTree:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   ::mleSection:editable := FALSE

RETURN self

METHOD IniTree:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   ::_IniTree:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

   // configure and set the context menu
   ::oCMenu := XbpMenu():new(::tv):create()
   ::oCMenu:addItem( {'~Delete'    , {|| ::nodeDelete() }} )

   ::tv:RbDown := {|mp1,mp2,oXbp| ::oCMenu:popUp(oXbp,mp1,2,;
                 XBPMENU_PU_DEFAULT + XBPMENU_PU_MOUSE_RBDOWN  ) }


   ::show()

RETURN self

METHOD IniTree:getItemsFromSection( cSection, oNode )
LOCAL xNode
LOCAL oItem
LOCAL i
LOCAL aList := {}
LOCAL n
LOCAL cCaption
LOCAL xValue

   ::oIni:readSectionValues( cSection, aList )

   FOR i := 1 TO LEN(aList)
      n = At('=', aList[i])
      cCaption := Left( aList[i], n-1 )
      xValue   := SubStr( aList[i], n+1 )

//*      oItem := XbpTreeViewItem():new()
//*      oItem:caption := cCaption
//*      oItem:setData( xValue )
//*      oItem:create()
//*      xNode := oNode:addItem( oItem )

      xNode := oNode:addItem(cCaption,,,,,xValue)

   NEXT

RETURN self

METHOD IniTree:updateControls( oItem, lMLE )
LOCAL aChild
LOCAL cBuff
LOCAL i
LOCAL cText
// Debug
LOCAL oParent

   DEFAULT lMLE to FALSE

   // oItem:EditBuffer() -> oItem:UndoBuffer
   //
   IF !(oItem:getData() == NIL)

      oParent   := oItem:getParentItem()

      ::sleSection:setData( oParent:caption )

      ::sleKey:setData(  oItem:caption )

      ::sleValue:setData(oItem:getdata() )

      IF lMLE
         ::updateControls(oParent)
      ENDIF

   ELSE

      aChild := oItem:getChildItems()

      cBuff := '['+oItem:caption+']'+CRLF
      for i := 1 to len(aChild)

         cText := aChild[i]:getData()
         IF cText = NIL
            cText := "NIL"
         ENDIF

         cBuff += aChild[i]:caption+'='+cText+CRLF
      next

      ::mleSection:setData(cBuff)
   endif

RETURN self

METHOD IniTree:set()
LOCAL oItem
LOCAL cSection
LOCAL cKey
LOCAL xValue
LOCAL cValue
LOCAL oParent

   oItem    := ::tv:getData()
   oParent  := oItem:getParentItem()
   cSection := oParent:caption
   cKey     := oItem:caption
   xValue   := oItem:getData()
   cValue   := AllTrim(::sleValue:getData())

   ::oIni:writeString( cSection, cKey, cValue )

   oItem:setData( cValue )

   // repaint
   //
   ::tv:setData(oParent)         // 1st focus on Section

   ::updateControls( oParent )

   ::tv:setData(oItem)           // 2nd to Child

RETURN self

METHOD IniTree:iSetItem()
   ::set()
   SetAppFocus(::tv)
RETURN self

METHOD IniTree:mleClick( aPos )
LOCAL c := ::mleSection:getData()
LOCAL n := ::mleSection:charFromLine()
LOCAL i := At(CRLF,c,n)
LOCAL cSection := SubStr( c, 2, AT(']',c)-2 )
LOCAL oItem
LOCAL aChild
LOCAL oObj

   n := iif( n == 0, Len(c), n )
   c := SubStr( c, n, i-n )
   c := left(c,At('=',c)-1)

   SetAppFocus(::tv)

   oItem := ::tv:rootItem
*   ::tv:lockUpdate(.T.)

*  aChild := oItem:getChildItems()               // macht nicht auf
   aChild := ::tv:getChildItems()                // geht beim 1st mal

*ALTD()

   for i := 1 to len(aChild)
      if aChild[i]:caption == cSection
         oItem := aChild[i]

         // need to "reset" focus
         //
         ::tv:setData(oItem)                     // focus on Section

         aChild := oItem:getChildItems()

         for n := 1 to len(aChild)
            if aChild[n]:caption == c
               ::tv:setData( aChild[n] )         // focus on item
               i := Len(aChild)+1
               oObj := ::tv:getData()
               exit
            endif
         next
      endif
   next
*   ::tv:lockUpdate(.F.)

RETURN self

METHOD IniTree:nodeDelete()
   local oNode := ::tv:getData()
   local lOk   := FALSE
   local cSection
   local cKey
   local oParent

   if oNode:getData() == NIL // section!
      cSection := oNode:caption
      if ConfirmBox(,'Delete complete section'+CRLF+CRLF+'['+cSection+']'+CRLF+CRLF+;
                    'Are You Sure?','Delete Section',,;
                    XBPMB_WARNING+XBPMB_APPMODAL) == XBPMB_RET_OK
         ::oIni:eraseSection(cSection)
         lOk := TRUE
      endif
   else
      cKey := oNode:caption
      cSection := oNode:getParentItem():caption
      if ConfirmBox(,'Delete key'+CRLF+CRLF+'['+cSection+']:'+cKey+CRLF+CRLF+;
                    'Are You Sure?','Delete Key',,;
                    XBPMB_WARNING+XBPMB_APPMODAL) == XBPMB_RET_OK
         ::oIni:deleteKey(cSection, cKey)
         lOk := TRUE
      endif
   endif
   if lOk
      ::tv:lockUpdate(.T.)
      oParent  := oNode:getparentItem()
      oParent:delItem(oNode)
      ::updateControls(::tv:getData(), TRUE)
      ::tv:lockUpdate(.F.)
   endif
   return self

METHOD IniTree:undo()
   local nH := FCreate(::oIni:fileName)

   if nH > 0
      FWrite(nH, ::cArchive)
      FClose(nH)
   endif
   return self


STATIC PROCEDURE OnDummy()                              // write into Debug Logfile
LOCAL iMax := PCount()
LOCAL i
LOCAL cText := ""

  FOR i := 1 TO iMax -1
      cText += Var2Char( PValue( i ) ) + CHR(9)
  NEXT
  cText += Var2Char( PValue( iMax ) )

  SET CONSOLE OFF
  SET ALTER ON

   ? cText

  SET ALTER OFF
  SET CONSOLE ON

RETURN


//EOF
/////